@@ -6,7 +6,7 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 #![crate_name="ndarray"]
-#![doc(html_root_url = "https://docs.rs/ndarray/0.10/")]
+#![doc(html_root_url = "https://docs.rs/ndarray/0.11/")]
 
 //! The `ndarray` crate provides an *n*-dimensional container for general elements
 //! and for numerics.
@@ -53,7 +53,7 @@
 //!   + Efficient floating point matrix multiplication even for very large
 //!     matrices; can optionally use BLAS to improve it further.
 //!   + See also the [`ndarray-parallel`] crate for integration with rayon.
-//! - **Requires Rust 1.18**
+//! - **Requires Rust 1.22**
 //!
 //! [`ndarray-parallel`]: https://docs.rs/ndarray-parallel
 //!
@@ -81,7 +81,9 @@ extern crate serde;
 extern crate rustc_serialize as serialize;
 
 #[cfg(feature="blas")]
-extern crate blas_sys;
+extern crate cblas_sys;
+#[cfg(feature="blas")]
+extern crate blas_src;
 
 extern crate matrixmultiply;
 
@@ -105,7 +107,7 @@ pub use dimension::NdIndex;
 pub use dimension::IxDynImpl;
 pub use indexes::{indices, indices_of};
 pub use error::{ShapeError, ErrorKind};
-pub use si::{Si, S};
+pub use slice::{Slice, SliceInfo, SliceNextDim, SliceOrIndex};
 
 use iterators::Baseiter;
 use iterators::{ElementsBase, ElementsBaseMut, Iter, IterMut};
@@ -144,7 +146,7 @@ mod free_functions;
 pub use free_functions::*;
 pub use iterators::iter;
 
-mod si;
+mod slice;
 mod layout;
 mod indexes;
 mod iterators;
@@ -422,21 +424,31 @@ pub type Ixs = isize;
 /// ## Slicing
 ///
 /// You can use slicing to create a view of a subset of the data in
-/// the array. Slicing methods include `.slice()`, `.islice()`,
-/// `.slice_mut()`.
+/// the array. Slicing methods include [`.slice()`], [`.slice_mut()`],
+/// [`.slice_move()`], and [`.slice_inplace()`].
 ///
 /// The slicing argument can be passed using the macro [`s![]`](macro.s!.html),
-/// which will be used in all examples. (The explicit form is a reference
-/// to a fixed size array of [`Si`]; see its docs for more information.)
+/// which will be used in all examples. (The explicit form is an instance of
+/// [`&SliceInfo`]; see its docs for more information.)
 ///
-/// [`Si`]: struct.Si.html
+/// [`&SliceInfo`]: struct.SliceInfo.html
+///
+/// If a range is used, the axis is preserved. If an index is used, a subview
+/// is taken with respect to the axis. See [*Subviews*](#subviews) for more
+/// information about subviews. Note that [`.slice_inplace()`] behaves like
+/// [`.subview_inplace()`] by preserving the number of dimensions.
+///
+/// [`.slice()`]: #method.slice
+/// [`.slice_mut()`]: #method.slice_mut
+/// [`.slice_move()`]: #method.slice_move
+/// [`.slice_inplace()`]: #method.slice_inplace
 ///
 /// ```
 /// // import the s![] macro
 /// #[macro_use(s)]
 /// extern crate ndarray;
 ///
-/// use ndarray::arr3;
+/// use ndarray::{arr2, arr3};
 ///
 /// fn main() {
 ///
@@ -457,8 +469,6 @@ pub type Ixs = isize;
 /// // - Every element in each row: `..`
 ///
 /// let b = a.slice(s![.., 0..1, ..]);
-/// // without the macro, the explicit argument is `&[S, Si(0, Some(1), 1), S]`
-///
 /// let c = arr3(&[[[ 1,  2,  3]],
 ///                [[ 7,  8,  9]]]);
 /// assert_eq!(b, c);
@@ -473,19 +483,42 @@ pub type Ixs = isize;
 /// let e = arr3(&[[[ 6,  5,  4]],
 ///                [[12, 11, 10]]]);
 /// assert_eq!(d, e);
+/// assert_eq!(d.shape(), &[2, 1, 3]);
+///
+/// // Let’s create a slice while taking a subview with
+/// //
+/// // - Both submatrices of the greatest dimension: `..`
+/// // - The last row in each submatrix, removing that axis: `-1`
+/// // - Row elements in reverse order: `..;-1`
+/// let f = a.slice(s![.., -1, ..;-1]);
+/// let g = arr2(&[[ 6,  5,  4],
+///                [12, 11, 10]]);
+/// assert_eq!(f, g);
+/// assert_eq!(f.shape(), &[2, 3]);
 /// }
 /// ```
 ///
 /// ## Subviews
 ///
-/// Subview methods allow you to restrict the array view while removing
-/// one axis from the array. Subview methods include `.subview()`,
-/// `.isubview()`, `.subview_mut()`.
+/// Subview methods allow you to restrict the array view while removing one
+/// axis from the array. Subview methods include [`.subview()`],
+/// [`.subview_mut()`], [`.into_subview()`], and [`.subview_inplace()`]. You
+/// can also take a subview by using a single index instead of a range when
+/// slicing.
 ///
 /// Subview takes two arguments: `axis` and `index`.
 ///
+/// [`.subview()`]: #method.subview
+/// [`.subview_mut()`]: #method.subview_mut
+/// [`.into_subview()`]: #method.into_subview
+/// [`.subview_inplace()`]: #method.subview_inplace
+///
 /// ```
-/// use ndarray::{arr3, aview2, Axis};
+/// #[macro_use(s)] extern crate ndarray;
+///
+/// use ndarray::{arr3, aview1, aview2, Axis};
+///
+/// # fn main() {
 ///
 /// // 2 submatrices of 2 rows with 3 elements per row, means a shape of `[2, 2, 3]`.
 ///
@@ -515,10 +548,15 @@ pub type Ixs = isize;
 ///
 /// assert_eq!(sub_col, aview2(&[[ 1,  4],
 ///                              [ 7, 10]]));
+///
+/// // You can take multiple subviews at once (and slice at the same time)
+/// let double_sub = a.slice(s![1, .., 0]);
+/// assert_eq!(double_sub, aview1(&[7, 10]));
+/// # }
 /// ```
 ///
-/// `.isubview()` modifies the view in the same way as `subview()`, but
-/// since it is *in place*, it cannot remove the collapsed axis. It becomes
+/// [`.subview_inplace()`] modifies the view in the same way as [`.subview()`],
+/// but since it is *in place*, it cannot remove the collapsed axis. It becomes
 /// an axis of length 1.
 ///
 /// `.outer_iter()` is an iterator of every subview along the zeroth (outer)
