@@ -1,6 +1,7 @@
 
 use Dimension;
 use {Shape, StrideShape};
+use dimension::IntoDimension;
 
 /// A trait for `Shape` and `D where D: Dimension` that allows
 /// customizing the memory layout (strides) of an array shape.
@@ -9,35 +10,40 @@ use {Shape, StrideShape};
 /// `Array::from_shape_vec`.
 pub trait ShapeBuilder {
     type Dim: Dimension;
+    type Strides;
 
+    fn into_shape(self) -> Shape<Self::Dim>;
     fn f(self) -> Shape<Self::Dim>;
     fn set_f(self, is_f: bool) -> Shape<Self::Dim>;
-    fn strides(self, strides: Self::Dim) -> StrideShape<Self::Dim>;
+    fn strides(self, strides: Self::Strides) -> StrideShape<Self::Dim>;
 }
 
 impl<D> From<D> for Shape<D>
-    where D: Dimension
+    where D: Dimension,
 {
-    fn from(d: D) -> Self {
-        Shape {
-            dim: d,
-            is_c: true,
-        }
+    /// Create a `Shape` from `dimension`, using the default memory layout.
+    fn from(dimension: D) -> Shape<D> {
+        dimension.into_shape()
     }
 }
 
-impl<D> From<D> for StrideShape<D>
-    where D: Dimension
+impl<T, D> From<T> for StrideShape<D>
+    where D: Dimension,
+          T: ShapeBuilder<Dim=D>,
 {
-    fn from(d: D) -> Self {
+    fn from(value: T) -> Self {
+        let shape = value.into_shape();
+        let d = shape.dim;
+        let st = if shape.is_c { d.default_strides() } else { d.fortran_strides() };
         StrideShape {
-            strides: d.default_strides(),
+            strides: st,
             dim: d,
             custom: false,
         }
     }
 }
 
+/*
 impl<D> From<Shape<D>> for StrideShape<D>
     where D: Dimension
 {
@@ -51,17 +57,25 @@ impl<D> From<Shape<D>> for StrideShape<D>
         }
     }
 }
+*/
 
-impl<D> ShapeBuilder for D
-    where D: Dimension
+impl<T> ShapeBuilder for T
+    where T: IntoDimension
 {
-    type Dim = D;
-    fn f(self) -> Shape<D> { self.set_f(true) }
-    fn set_f(self, is_f: bool) -> Shape<D> {
-        Shape::from(self).set_f(is_f)
+    type Dim = T::Dim;
+    type Strides = T;
+    fn into_shape(self) -> Shape<Self::Dim> {
+        Shape {
+            dim: self.into_dimension(),
+            is_c: true,
+        }
     }
-    fn strides(self, st: D) -> StrideShape<D> {
-        Shape::from(self).strides(st)
+    fn f(self) -> Shape<Self::Dim> { self.set_f(true) }
+    fn set_f(self, is_f: bool) -> Shape<Self::Dim> {
+        self.into_shape().set_f(is_f)
+    }
+    fn strides(self, st: T) -> StrideShape<Self::Dim> {
+        self.into_shape().strides(st.into_dimension())
     }
 }
 
@@ -69,6 +83,8 @@ impl<D> ShapeBuilder for Shape<D>
     where D: Dimension
 {
     type Dim = D;
+    type Strides = D;
+    fn into_shape(self) -> Shape<D> { self }
     fn f(self) -> Self { self.set_f(true) }
     fn set_f(mut self, is_f: bool) -> Self {
         self.is_c = !is_f;
@@ -84,3 +100,11 @@ impl<D> ShapeBuilder for Shape<D>
 }
 
 
+impl<D> Shape<D>
+    where D: Dimension,
+{
+    // Return a reference to the dimension
+    //pub fn dimension(&self) -> &D { &self.dim }
+    /// Return the size of the shape in number of elements
+    pub fn size(&self) -> usize { self.dim.size() }
+}
